/*
 * DataIO Library for ESP32 I2C Sensor access
 *
 * (c) 2020 Matthias Thiele
 */
package de.mmth.dataio.csv;

import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.List;

/**
 * Create CSV Files from sensor values.
 * 
 * @author matthias
 */
public class CsvFile implements Closeable {
  private final List<Column> columns = new ArrayList<>();
  private Path csvFile;
  private boolean isNext = false;
  private BufferedWriter writer = null;
  
  /**
   * Constructor with file name. If there is only a
   * name without path, then the file will be created
   * in the temp directory.
   * 
   * @param name 
   */
  public CsvFile(String name) {
    if (name != null) {
      Path path;

      try {
        if (name.contains("/") || name.contains("\\")) {
          path = Files.createFile(new File(name).toPath());
        } else {
          path = Files.createTempFile(name, ".csv");
        }

        isNext = path.toFile().length() > 0;
        writer = Files.newBufferedWriter(path, StandardOpenOption.CREATE);
      } catch (IOException ex) {
        path = null;
      }

      csvFile = path;
    }
  }
  
  @Override
  public void close() throws IOException {
    if (writer != null) {
      writer.close();
    }
  }
  
  /**
   * Get the complete file path of the CSV file.
   * 
   * @return 
   */
  public Path getPath() {
    return csvFile;
  }
  
  /**
   * Add a column to the CSV file.
   * 
   * @param nextColumn 
   */
  public void addColumn(Column nextColumn) {
    columns.add(nextColumn);
  }
  
  /**
   * Format a row from the column definitions
   * and the supported values. If there are
   * less colum definitions or less values, 
   * only the minimum of both will be evaluated.
   * 
   * @param values
   * @return 
   */
  public String formatRow(int ...values) {
    StringBuilder buf = new StringBuilder(200);
    int cols = Math.min(values.length, columns.size());
    
    for (int i = 0; i < cols; i++) {
      if (i > 0) {
        buf.append(", ");
      }
      
      buf.append(columns.get(i).format(values[i]));
    }
    
    return buf.toString();
  }
  
  /**
   * Formats the values List with the column definitions
   * and adds this line to the CSV file.
   * 
   * @param values
   * @throws IOException 
   */
  public void addRow(int ...values) throws IOException {
    if (csvFile == null) {
      throw new IOException("Missing csv file");
    }
    
    String row = formatRow(values);
    writer.append(isNext ? ("\r\n" + row) : row);
    isNext = true;
  }
}
